@@ -225,7 +225,12 @@ class EC2Connection(ConnectionUserAndKey):
                          urllib.quote(params[key], safe='-_~'))
 
         qs = '&'.join(pairs)
-        string_to_sign = '\n'.join(('GET', self.host, path, qs))
+
+        hostname = self.host
+        if (self.secure and self.port != 443) or (not self.secure and self.port != 80):
+            hostname += ":" + str(self.port)
+
+        string_to_sign = '\n'.join(('GET', hostname, path, qs))
 
         b64_hmac = base64.b64encode(
             hmac.new(secret_key, string_to_sign, digestmod=sha256).digest()
@@ -266,6 +271,7 @@ class EC2NodeDriver(NodeDriver):
     path = '/'
 
     _instance_types = EC2_US_EAST_INSTANCE_TYPES
+    features = {'create_node': ['ssh_key']}
 
     NODE_STATE_MAP = {
         'pending': NodeState.PENDING,
@@ -319,15 +325,20 @@ class EC2NodeDriver(NodeDriver):
 
         name = tags.get('Name', instance_id)
 
+        public_ip = findtext(element=element, xpath='ipAddress',
+                              namespace=NAMESPACE)
+        public_ips = [public_ip] if public_ip else []
+        private_ip = findtext(element=element, xpath='privateIpAddress',
+                                 namespace=NAMESPACE)
+        private_ips = [private_ip] if private_ip else []
+
         n = Node(
             id=findtext(element=element, xpath='instanceId',
                         namespace=NAMESPACE),
             name=name,
             state=state,
-            public_ip=[findtext(element=element, xpath='ipAddress',
-                                namespace=NAMESPACE)],
-            private_ip=[findtext(element=element, xpath='privateIpAddress',
-                                 namespace=NAMESPACE)],
+            public_ip=public_ips,
+            private_ip=private_ips,
             driver=self.connection.driver,
             extra={
                 'dns_name': findattr(element=element, xpath="dnsName",
@@ -381,7 +392,40 @@ class EC2NodeDriver(NodeDriver):
                                   namespace=NAMESPACE),
                       name=findtext(element=element, xpath='imageLocation',
                                     namespace=NAMESPACE),
-                      driver=self.connection.driver)
+                      driver=self.connection.driver,
+                      extra={
+                          'state': findattr(element=element,
+                                            xpath="imageState",
+                                            namespace=NAMESPACE),
+                          'ownerid': findattr(element=element,
+                                        xpath="imageOwnerId",
+                                        namespace=NAMESPACE),
+                          'owneralias': findattr(element=element,
+                                        xpath="imageOwnerAlias",
+                                        namespace=NAMESPACE),
+                          'ispublic': findattr(element=element,
+                                        xpath="isPublic",
+                                        namespace=NAMESPACE),
+                          'architecture': findattr(element=element,
+                                        xpath="architecture",
+                                        namespace=NAMESPACE),
+                          'imagetype': findattr(element=element,
+                                        xpath="imageType",
+                                        namespace=NAMESPACE),
+                          'platform': findattr(element=element,
+                                        xpath="platform",
+                                        namespace=NAMESPACE),
+                          'rootdevicetype': findattr(element=element,
+                                        xpath="rootDeviceType",
+                                        namespace=NAMESPACE),
+                          'virtualizationtype': findattr(element=element,
+                                        xpath="virtualizationType",
+                                        namespace=NAMESPACE),
+                          'hypervisor': findattr(element=element,
+                                        xpath="hypervisor",
+                                        namespace=NAMESPACE)
+                      }
+        )
         return n
 
     def list_nodes(self):
@@ -398,7 +442,8 @@ class EC2NodeDriver(NodeDriver):
 
         nodes_elastic_ips_mappings = self.ex_describe_addresses(nodes)
         for node in nodes:
-            node.public_ip.extend(nodes_elastic_ips_mappings[node.id])
+            ips = nodes_elastic_ips_mappings[node.id]
+            node.public_ip.extend(ips)
         return nodes
 
     def list_sizes(self, location=None):
@@ -692,6 +737,15 @@ class EC2NodeDriver(NodeDriver):
         self.connection.request(self.path,
                                 params=params.copy()).object
 
+    def _add_instance_filter(self, params, node):
+        """
+        Add instance filter to the provided params dictionary.
+        """
+        params.update({
+            'Filter.0.Name': 'instance-id',
+            'Filter.0.Value.0': node.id
+        })
+
     def ex_describe_addresses(self, nodes):
         """
         Return Elastic IP addresses for all the nodes in the provided list.
@@ -709,10 +763,7 @@ class EC2NodeDriver(NodeDriver):
         params = {'Action': 'DescribeAddresses'}
 
         if len(nodes) == 1:
-            params.update({
-                'Filter.0.Name': 'instance-id',
-                'Filter.0.Value.0': nodes[0].id
-            })
+            self._add_instance_filter(params, nodes[0])
 
         result = self.connection.request(self.path,
                                          params=params.copy()).object
@@ -859,7 +910,12 @@ class EC2NodeDriver(NodeDriver):
 
         for node in nodes:
             tags = {'Name': kwargs['name']}
-            self.ex_create_tags(node=node, tags=tags)
+
+            try:
+                self.ex_create_tags(node=node, tags=tags)
+            except Exception:
+                continue
+
             node.name = kwargs['name']
             node.extra.update({'tags': tags})
 
@@ -1012,6 +1068,12 @@ class EucNodeDriver(EC2NodeDriver):
         raise NotImplementedError(
                 'list_locations not implemented for this driver')
 
+    def _add_instance_filter(self, params, node):
+        """
+        Eucalyptus driver doesn't support filtering on instance id so this is a
+        no-op.
+        """
+        pass
 
 # Nimbus clouds have 3 EC2-style instance types but their particular RAM
 # allocations are configured by the admin
