@@ -54,6 +54,8 @@ class Provider(object):
     @cvar NIMBUS: Nimbus
     @cvar BLUEBOX: Bluebox
     @cvar OPSOURCE: Opsource Cloud
+    @cvar NINEFOLD: Ninefold
+    @cvar TERREMARK: Terremark
     """
     DUMMY = 0
     EC2 = 1  # deprecated name
@@ -91,6 +93,8 @@ class Provider(object):
     OPENSTACK = 31
     SKALICLOUD = 32
     SERVERLOVE = 33
+    NINEFOLD = 34
+    TERREMARK = 35
 
 class NodeState(object):
     """
