@@ -161,7 +161,9 @@ class S3StorageDriver(StorageDriver):
     name = 'Amazon S3 (standard)'
     connectionCls = S3Connection
     hash_type = 'md5'
+    supports_chunked_encoding = False
     ex_location_name = ''
+    namespace = NAMESPACE
 
     def list_containers(self):
         response = self.connection.request('/')
@@ -300,11 +302,17 @@ class S3StorageDriver(StorageDriver):
 
     def upload_object_via_stream(self, iterator, container, object_name,
                                  extra=None, ex_storage_class=None):
-        # Amazon S3 does not support chunked transfer encoding.
-        # Using multipart upload to "emulate" it would mean unnecessary
-        # buffering of data in memory.
-        raise NotImplementedError(
-            'upload_object_via_stream not implemented for this driver')
+        # Amazon S3 does not support chunked transfer encoding so the whole data
+        # is read into memory before uploading the object.
+        upload_func = self._upload_data
+        upload_func_kwargs = {}
+
+        return self._put_object(container=container, object_name=object_name,
+                                upload_func=upload_func,
+                                upload_func_kwargs=upload_func_kwargs,
+                                extra=extra, iterator=iterator,
+                                verify_hash=False,
+                                storage_class=ex_storage_class)
 
     def delete_object(self, obj):
         object_name = self._clean_object_name(name=obj.name)
@@ -337,7 +345,7 @@ class S3StorageDriver(StorageDriver):
             objects = self._to_objs(obj=response.object,
                                        xpath='Contents', container=container)
             is_truncated = response.object.findtext(fixxpath(xpath='IsTruncated',
-                                                   namespace=NAMESPACE)).lower()
+                                                   namespace=self.namespace)).lower()
             exhausted = (is_truncated == 'false')
 
             if (len(objects) > 0):
@@ -407,21 +415,21 @@ class S3StorageDriver(StorageDriver):
 
     def _to_containers(self, obj, xpath):
         return [ self._to_container(element) for element in \
-                 obj.findall(fixxpath(xpath=xpath, namespace=NAMESPACE))]
+                 obj.findall(fixxpath(xpath=xpath, namespace=self.namespace))]
 
     def _to_objs(self, obj, xpath, container):
         return [ self._to_obj(element, container) for element in \
-                 obj.findall(fixxpath(xpath=xpath, namespace=NAMESPACE))]
+                 obj.findall(fixxpath(xpath=xpath, namespace=self.namespace))]
 
     def _to_container(self, element):
         extra = {
             'creation_date': findtext(element=element, xpath='CreationDate',
-                                      namespace=NAMESPACE)
+                                      namespace=self.namespace)
         }
 
         container = Container(
                         name=findtext(element=element, xpath='Name',
-                                      namespace=NAMESPACE),
+                                      namespace=self.namespace),
                         extra=extra,
                         driver=self
                     )
@@ -441,19 +449,19 @@ class S3StorageDriver(StorageDriver):
 
     def _to_obj(self, element, container):
         owner_id = findtext(element=element, xpath='Owner/ID',
-                            namespace=NAMESPACE)
+                            namespace=self.namespace)
         owner_display_name = findtext(element=element,
                                       xpath='Owner/DisplayName',
-                                      namespace=NAMESPACE)
+                                      namespace=self.namespace)
         meta_data = { 'owner': { 'id': owner_id,
                                  'display_name':owner_display_name }}
 
         obj = Object(name=findtext(element=element, xpath='Key',
-                                   namespace=NAMESPACE),
+                                   namespace=self.namespace),
                      size=int(findtext(element=element, xpath='Size',
-                                       namespace=NAMESPACE)),
+                                       namespace=self.namespace)),
                      hash=findtext(element=element, xpath='ETag',
-                                   namespace=NAMESPACE).replace('"', ''),
+                                   namespace=self.namespace).replace('"', ''),
                      extra=None,
                      meta_data=meta_data,
                      container=container,
